#!c:/perl/bin/perl -w

# standard usage by username/pagename, first examples depend on default options.
#perl fetchCapcToFile.pl -h yourCAPCHost -t "Network Device Health"
#perl fetchCapcToFile.pl -h yourCAPCHost -t "Network Device Health" -a timeRange=4
#perl fetchCapcToFile.pl -h yourCAPCHost -t "Network Device Health" -a timeRange=4 -u JSmith -p Y0urP8ssWd -P 80
# ( EACH of above will save-as & overwrite ./Network_Device_Health.pdf )

# OR alternate usage by pg= IDs (bypasses mysql-lookup)
#perl fetchCapcToFile.pl -h yourCAPCHost -t 2000002 -a timeRange=4
# ( Same page as first section, except now ./2000002.pdf as a filename )

# OR advanced options for context and other user selection-changes
#perl fetchCapcToFile.pl -h yourCAPCHost -t "Router Performance" -a "DeviceID=5"
#perl fetchCapcToFile.pl -h yourCAPCHost -t "Router Performance" -a "DeviceID=5&timeRange=12"

# other notes
#   last 7 days --> timeRange=4
#   previous week --> timeRange=11
#   last 30 days --> timeRange=6
#   previous month --> timeRange=12


#####################################################
# Libraries
#####################################################

# On New perl-setup...
# ppm install DBD-mysql

# On New Strawberry...
#  ..."the way to fix the problem with Strawberry Perl is to"...
#     copy C:\strawberry\c\bin\libmysql_.dll C:\strawberry\perl\vendor\lib\auto\DBD\mysql\libmysql_.dll 
# (not tested beyond a mysql-nqportal connection)


use warnings;
use strict;
use DBI; 
use DBD::mysql; 
use Getopt::Std;
use LWP::Debug qw(+);
use LWP::Simple;
#use Time::Local;
use HTTP::Response;
 use HTTP::Request;
 use LWP::UserAgent;
 use HTTP::Cookies;
 use HTTP::Request::Common;
use Compress::Zlib;

#####################################################
# Variable Declarations 
#####################################################

#Global Database Connection Variables 
my $driver = "mysql";
my $db_usr = "netqos";
my $pt_local = "3306";
my $db_pw_local = "netqos";
my $db = "netqosportal";

#CAPC Connection Variables
my $host = "localhost";
my $port = "8181";

#SQL selection variables 
my $sql=''; 
my $dbh='';
my $sth='';


#####################################################
# Main Program 
#####################################################

#Command Line Parameters 
my %cmdLineOptions = ();
my $userName='admin';
my $password='admin';
my $pageTitle='Routers/Switches Overview';
my $filename='';
my $args='';

#Get Command line options 
getopts("u:p:t:h:P:a:", \%cmdLineOptions);

# -u option
# User Name
if ( defined $cmdLineOptions{u} ) {
    $userName = $cmdLineOptions{u};
}

# -p option
# PassWord
if ( defined $cmdLineOptions{p} ) {
    $password = $cmdLineOptions{p};
}

# -t option
# Title of users myPage report
if ( defined $cmdLineOptions{t} ) {
    $pageTitle = $cmdLineOptions{t};
}

# -h option
# host server
if ( defined $cmdLineOptions{h} ) {
    $host = $cmdLineOptions{h};
}

# -P option
# wep PORT server
if ( defined $cmdLineOptions{P} ) {
    $port = $cmdLineOptions{P};
}

# -a option
# args (such as an a group-path or device-id, protocol-id for RA...)
if ( defined $cmdLineOptions{a} ) {
    $args = '&'.$cmdLineOptions{a};
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Start processing 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#push(@LWP::Prococol::http::EXTRA_SOCK_OPTS, SendTE => 0);

my $pageid;
if ($pageTitle  =~ /^(\d+)/ ) {    # match valid number, from ref for real =~ /^(\d+\.?\d*|\.\d+)$/  
    $pageid = $pageTitle;
}
else {
   
# connect to CAPC database
my $dsn = "DBI:$driver:database=$db;host=$host;port=$pt_local;";
$dbh = DBI->connect($dsn, $db_usr, $db_pw_local, { RaiseError => 1, PrintError => 0 } ) or die "Couldnt connect to local database: $DBI::errstr\n";

# query for a mating User-Page ID
$sql = qq{ select pageID from ( 
    select pageid, pagetitle, userid from page_info2
  union
    select pageid, pagetitle, userid from page_info2_i18n
 ) upages
 where pageTitle='$pageTitle'
   and (userid = 0 or userid in (select userid from user_definitions where name='$userName' ) )
 order by userid desc limit 1 };
#warn "DEBUG - sql query: " . $sql;
$sth = $dbh->prepare( $sql );
$sth->execute();
  ($pageid) = $sth->fetchrow_array();
$sth->finish;

#exit on page-not-found
if ( defined($pageid) eq 0 ) {
        die "Page title not found : " . $pageTitle ;
        }

} 

# begin web-get processing

my $pageURL = qq{ http://$host:$port/pc/desktop/page?pg=$pageid&portrait=1&print=1$args };
		# &mi=2
my $pdfURL = $pageURL;
$pdfURL =~ s/desktop/pdf/;
print $pageURL."\n";

my $loginURL = qq{ http://$host:8381/sso/sign-in-process.jsp };
print $loginURL."\n";


my $cookie_jar = HTTP::Cookies->new(file=>"cookies.dat");
my $ua = LWP::UserAgent->new( keep_alive => 1 );
	# [ keep_alive => 1, SendTE => 0 ]  
	# LWP::UserAgent->new();
$ua->agent("Mozilla/5.0 (Windows NT 6.1; WOW64) Chrome/47.0.2526.111 Safari/537.36");
$ua->timeout(600);
$ua->cookie_jar($cookie_jar);

#  Wise Monk cookie-tip = http://www.perlmonks.org/?node_id=168363
# following line looks beneficial, but doesn't appear to work...
# $ua->cookie_jar(HTTP::Cookies->new(file => 'cookies.dat', autosave =>1));

warn "Get a jsession-cookie from the user page";
#my $request = HTTP::Request->new(GET=>$loginURL);
#$cookie_jar->add_cookie_header($request);
#$cookie_jar->add_cookie_header($doUserPage);
#warn "request the user page";
my $response = $ua->request(GET $pageURL);
#warn "extract sessionid";
$cookie_jar->extract_cookies($response);

#my $content = $response->content();
# To Check login-page...
#    print "Content: $content \n";


warn "Now post the login form...";
#  $ua->default_header('Accept' => 'image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*\r\n');
#$ua->default_header('Referer' => "$myURL\r\n");
#$ua->default_header('Accept' => 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8');
#$ua->default_header('Accept-Encoding' => 'gzip,deflate');
#$request = GET "$loginURL";
#$request->accept_decodable;
#$ua->default_header('Accept-Language' => 'en-US,en;q=0.8');
my $doLogin = POST $loginURL, 
          [          txtSsoProductCode => 'pc',
		     txtUsername => $userName,
                     txtPassword => $password,
		     txtSsoRedirectUrl => $pageURL
	];
$cookie_jar->add_cookie_header($doLogin);
$doLogin->header(Accept => 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8');
$doLogin->referer( $pageURL );
$response = $ua->request($doLogin);
if ($response->code eq 302) {
	$cookie_jar->extract_cookies($response);
} else {
	die "POST error is " . $response->status_line ;
}

warn "Now submit and save the final printable request...";
my $doUserPage = GET $pdfURL;
$cookie_jar->add_cookie_header($doUserPage);
$doUserPage->header(Accept => 'image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*');
$doUserPage->header(AcceptEncoding => 'gzip,deflate');
$response = $ua->request($doUserPage);

  # check the outcome
  if ($response->redirects > 0) { die "Invalid User/Password from ($userName/$password) ? "; }
  if ($response->is_success) {
     my $pdfhandle;
     $filename = $pageTitle;
     $filename =~ s/[\\]//;
     $filename =~ s/[\/]//;
     $filename =~ s/[ ]/_/g;
     open( $pdfhandle, ">".$filename.".pdf");
     binmode($pdfhandle);
     print $pdfhandle $response->decoded_content; 
     	# print $pdfhandle $response->content; 
     close $pdfhandle;
  } else {
     print "Error: " . $response->status_line . "\n";
     # print $response->content; 
  }

#end 

